# -*- mode: Makefile; tab-width: 4; -*-
# ex: ts=4
#
# $FreeBSD: ports/Mk/bsd.python3.mk,v 1.129 2011/03/05 00:39:33 bf Exp $
#

#.if !defined(_POSTMKINCLUDED) && !defined(Python3_Pre_Include)

#Python_Pre_Include=		bsd.python3.mk
#Python_Include_MAINTAINER=	python@FreeBSD.org

.include <../../Mk/bsd.commands.mk>
#TAIL=	/usr/bin/tail

# This file contains some variable definitions that are supposed to
# make your life easier when dealing with ports related to the Python
# language. It's automatically included when USE_PYTHON3 or PYTHON3_VERSION
# is defined in the ports' makefile. Define PYTHON3_VERSION to override the
# defaults that USE_PYTHON3 would give you. If your port requires only some
# set of Python versions, you can define USE_PYTHON3 as [min]-[max] or
# min+. (eg. 2.1-2.3, 2.0+ or -2.2)
#
# The variables:
#
# PYTHON3BASE		- Python port's installation prefix.
#				  default: ${LOCALBASE}
#
# PYTHON3_CMD		- Python's command line file name, including the version
#				  number (used for dependencies).
#				  default: ${PYTHON3BASE}/bin/${PYTHON3_VERSION}
#
# PYTHON3_DISTFILE	- The ${DISTFILE} for your python version. Needed for
#				  extensions like Tkinter, py-gdbm and py-expat, which
#				  are built from sources contained in the Python distribution.
#
# PYTHON3_MASTER_SITES
#			- The ${MASTER_SITES} for your python version. (You must
#				  use this instead of ${MASTER_SITE_PYTHON3} to support
#				  python-devel port.)
#
# PYTHON3_MASTER_SITE_SUBDIR
#			- The ${MASTER_SITE_SUBDIR} for your python version.
#
# PYTHON3_INCLUDEDIR	- Location of the Python include files.
#				  default: ${PYTHON3BASE}/include/${PYTHON3_VERSION}
#
# PYTHON3_LIBDIR	- Base of the python library tree
#				  default: ${PYTHON3BASE}/lib/${PYTHON3_VERSION}
#
# PYTHON3_PKGNAMEPREFIX
#			- Use this as a ${PKGNAMEPREFIX} to distinguish
#				  packages for different Python versions.
#				  default: py${PYTHON3_SUFFIX}-
#
# PYTHON3_PKGNAMESUFFIX
#			- If your port's name is more popular without `py-'
#				  prefix, use this as a ${PKGNAMESUFFIX} alternatively.
#				  default: -py${PYTHON3_SUFFIX}
#
# PYTHON3_PLATFORM	- Python's idea of the OS release.
#				  XXX This is faked with ${OPSYS} and ${OSREL} until I
#				  find out how to delay defining a variable until after
#				  a certain target has been built.
#
# PYTHON3_PORTSDIR	- The source of your binary's port. Needed for the RUN_DEPENDS.
#
# PYTHON3_PORTVERSION
#			- Version number suitable for ${PORTVERSION}.
#
# PYTHON3_REL		- Version number in numerical format, to ease comparison 
#   				  in makefiles
#
# PYTHON3_SITELIBDIR	- Location of the site-packages tree. Don't change,
#				  unless you know what you do.
#				  default: ${PYTHON3_LIBDIR}/site-packages
#
# PYTHON3_SUFFIX	- Yet another short version number, primarily intended
#				  for ${PYTHON3_PKGNAMEPREFIX}.
#
# PYTHON3_VERSION	- Version of the python binary in your ${PATH}, in the
#				  format "python2.0". Set this in your makefile in case you
#				  want to build extensions with an older binary.
#				  default: depends on the version of your python binary
#
# PYTHON3_DEFAULT_VERSION
#			- Version of the default python binary in your ${PATH}, in
#				  the format "python2.7". Set this in your /etc/make.conf
#				  in case you want to use an older version as a default.
#				  default: python2.7
#
# PYTHON3_WRKSRC	- The ${WRKSRC} for your python version. Needed for
#				  extensions like Tkinter, py-gdbm and py-expat, which
#				  are built from sources contained in the Python
#				  distribution.
#
# There are PREFIX-clean variants of the PYTHON3_*DIR variables above.
# They are meant to be used in the installation targets.
#
# PYTHON3PREFIX_INCLUDEDIR:	default: ${PREFIX}/include/${PYTHON3_VERSION}
# PYTHON3PREFIX_LIBDIR:		default: ${PREFIX}/lib/${PYTHON3_VERSION}
# PYTHON3PREFIX_SITELIBDIR:	default: ${PYTHON3PREFIX_LIBDIR}/site-packages
#
# PY3DISTUTILS		- Dependency line for the distutils extension. As of
#				  Python-2.0, the distutils are in the base distribution.
#
# USE_PYTHON3_PREFIX	- Says that the port installs in ${PYTHON3BASE}.
#
# USE_PY3DISTUTILS	- Use distutils as do-configure, do-build and do-install
#					  targets.
#
# PY3SETUP		- Name of the setup script used by the distutils package.
#					  default: setup.py
#
# PY3DISTUTILS_PKGNAME
#			- Internal name in the distutils for egg-info.
#				  default: ${PORTNAME}
#
# PY3DISTUTILS_PKGVERSION
#			- Internal version in the distutils for egg-info.
#				  default: ${PORTVERSION}
#
# PY3DISTUTILS_CONFIGURE_TARGET
#			- Pass this command to distutils on configure stage.
#				  default: config
#
# PY3DISTUTILS_BUILD_TARGET
#			- Pass this command to distutils on build stage.
#				  default: build
#
# PY3DISTUTILS_INSTALL_TARGET
#			- Pass this command to distutils on install stage.
#				  default: install
#
# PY3DISTUTILS_CONFIGUREARGS
#			- Arguments to config with distutils.
#				  default: <empty>
#
# PY3DISTUTILS_BUILDARGS
#			- Arguments to build with distutils.
#				  default: <empty>
#
# PY3DISTUTILS_INSTALLARGS
#			- Arguments to install with distutils.
#				  default: -c -O1 --prefix=${PREFIX}
#
# PY3DISTUTILS_EGGINFO
#			- Canonical name for egg-info.
#				  default: ${PYDISTUTILS_PKGNAME:C/[^A-Za-z0-9.]+/_/g}-${PYDISTUTILS_PKGVERSION:C/[^A-Za-z0-9.]+/_/g}-${PYTHON3_VERSION:S/thon//}.egg-info
#
# PY3DISTUTILS_NOEGGINFO
#			- Skip an egg-info entry from plist when defined.
#

_PYTHON3_PORTBRANCH=		3.2
_PYTHON3_ALLBRANCHES=		3.2 3.1# preferred first

# Determine version number of Python to use
.if !defined(PYTHON3_DEFAULT_VERSION)
. if exists(${LOCALBASE}/bin/python)
_PYTHON3_DEFAULT_VERSION!=	(${LOCALBASE}/bin/python -c \
				'import sys; print(sys.version[:3])' 2> /dev/null \
				|| ${ECHO_CMD} ${_PYTHON3_PORTBRANCH}) | ${TAIL} -1
. else
_PYTHON3_DEFAULT_VERSION=	${_PYTHON3_PORTBRANCH}
. endif
PYTHON3_DEFAULT_VERSION=	python${_PYTHON3_DEFAULT_VERSION}
.endif

.if defined(PYTHON3_VERSION)
_PYTHON3_VERSION:=	${PYTHON3_VERSION:S/^python//}
_PYTHON3_CMD=		${LOCALBASE}/bin/${PYTHON3_VERSION}
.else
_PYTHON3_VERSION:=	${PYTHON3_DEFAULT_VERSION:S/^python//}
_PYTHON3_CMD=		${LOCALBASE}/bin/${PYTHON3_DEFAULT_VERSION}
.endif

.if !defined(USE_PYTHON3)
.if defined(USE_PYTHON3_BUILD)
USE_PYTHON3=		${USE_PYTHON3_BUILD}
.elif defined(USE_PYTHON3_RUN)
USE_PYTHON3=		${USE_PYTHON3_RUN}
.else
USE_PYTHON3=		any
.endif	# defined(USE_PYTHON3_BUILD)
.else
USE_PYTHON3_BUILD=	yes
USE_PYTHON3_RUN=		yes
.endif	# !defined(USE_PYTHON3)

# Validate Python version whether it meets USE_PYTHON3 version restriction.
_PYTHON3_VERSION_CHECK:=			${USE_PYTHON3:C/^([1-9]\.[0-9])$/\1-\1/}
_PYTHON3_VERSION_MINIMUM_TMP:=	${_PYTHON3_VERSION_CHECK:C/([1-9]\.[0-9])[-+].*/\1/}
_PYTHON3_VERSION_MINIMUM:=		${_PYTHON3_VERSION_MINIMUM_TMP:M[1-9].[0-9]}
_PYTHON3_VERSION_MAXIMUM_TMP:=	${_PYTHON3_VERSION_CHECK:C/.*-([1-9]\.[0-9])/\1/}
_PYTHON3_VERSION_MAXIMUM:=		${_PYTHON3_VERSION_MAXIMUM_TMP:M[1-9].[0-9]}

.if !empty(_PYTHON3_VERSION_MINIMUM) && ( \
		${_PYTHON3_VERSION} < ${_PYTHON3_VERSION_MINIMUM})
_PYTHON3_VERSION_NONSUPPORTED=	${_PYTHON3_VERSION_MINIMUM} at least
.elif !empty(_PYTHON3_VERSION_MAXIMUM) && ( \
		${_PYTHON3_VERSION} > ${_PYTHON3_VERSION_MAXIMUM})
_PYTHON3_VERSION_NONSUPPORTED=	${_PYTHON3_VERSION_MAXIMUM} at most
.endif

# If we have an unsupported version of Python, try another.
.if defined(_PYTHON3_VERSION_NONSUPPORTED)
.if defined(PYTHON3_VERSION) || defined(PYTHON3_CMD)
IGNORE=		needs Python ${_PYTHON3_VERSION_NONSUPPORTED}.\
		But you specified ${_PYTHON3_VERSION}
.else
.undef _PYTHON3_VERSION
.for ver in ${_PYTHON3_ALLBRANCHES}
__VER=		${ver}
.if !defined(_PYTHON3_VERSION) && \
	!(!empty(_PYTHON3_VERSION_MINIMUM) && ( \
		${__VER} < ${_PYTHON3_VERSION_MINIMUM})) && \
	!(!empty(_PYTHON3_VERSION_MAXIMUM) && ( \
		${__VER} > ${_PYTHON3_VERSION_MAXIMUM}))
_PYTHON3_VERSION=	${ver}
_PYTHON3_CMD=		${LOCALBASE}/bin/python${ver}
.endif
.endfor
.if !defined(_PYTHON3_VERSION)
IGNORE=				needs an unsupported version of Python
_PYTHON3_VERSION=	${_PYTHON3_PORTBRANCH} # just to avoid version sanity checking.
.endif
.endif	# defined(PYTHON3_VERSION) || defined(PYTHON3_CMD)
.endif	# defined(_PYTHON3_VERSION_NONSUPPORTED)

PYTHON3_VERSION?=	python${_PYTHON3_VERSION}
PYTHON3_CMD?=		${_PYTHON3_CMD}
.if !defined(PYTHON3BASE)
PYTHON3BASE!=		(${PYTHON3_CMD} -c 'import sys; print sys.prefix' \
			2> /dev/null || ${ECHO_CMD} ${LOCALBASE}) | ${TAIL} -1
.endif
DEPENDS_ARGS+=		PYTHON3_VERSION=${PYTHON3_VERSION}

# We can only use the cached version if we are using the default python version.  Otherwise it
# should point to some other version we have installed, according to the port USE_PYTHON3
# specification
.if !defined(PYTHON3_DEFAULT_PORTVERSION) || (${PYTHON3_VERSION} != ${PYTHON3_DEFAULT_VERSION})
_PYTHON3_PORTVERSION!=	(${PYTHON3_CMD} -c 'import string, sys; \
			print string.split(sys.version)[0].replace("b",".b")' 2> /dev/null) | ${TAIL} -1
.if !defined(PYTHON3_NO_DEPENDS) && !empty(_PYTHON3_PORTVERSION)
PYTHON3_PORTVERSION=	${_PYTHON3_PORTVERSION}
.endif
.elif defined(PYTHON3_DEFAULT_PORTVERSION)
PYTHON3_PORTVERSION=	${PYTHON3_DEFAULT_PORTVERSION}
.endif

# Propagate the chosen python version to submakes.
.MAKEFLAGS:	PYTHON3_VERSION=python${_PYTHON3_VERSION}

# Python-3.2
.if ${PYTHON3_VERSION} == "python3.2"
PYTHON3_PORTVERSION?=3.2
PYTHON3_PORTSDIR=	${PORTSDIR}/lang/python32
PYTHON3_REL=			320
PYTHON3_SUFFIX=		32
PYTHON3_VER=			3.2

# Python-3.1
.elif ${PYTHON3_VERSION} == "python3.1"
PYTHON3_PORTVERSION?=3.1.3
PYTHON3_PORTSDIR=	${PORTSDIR}/lang/python31
PYTHON3_REL=			313
PYTHON3_SUFFIX=		31
PYTHON3_VER=			3.1

.elif defined(FORCE_PYTHON3_VERSION)
PYTHON3_PORTSDIR=	# empty
PYTHON3_NO_DEPENDS=	YES
PYTHON3_REL!=		${PYTHON3_CMD} -c 'import sys; h = "%x" % sys.hexversion; \
						print h[0]+h[2]+h[4]'
PYTHON3_SUFFIX!=		${PYTHON3_CMD} -c 'import sys; h = "%x" % sys.hexversion; \
						print h[0]+h[2]'
PYTHON3_VER!=		${PYTHON3_CMD} -c 'import sys; print sys.version[:3]'

.else
check-makevars::
	@${ECHO} "Makefile error: bad value for PYTHON3_VERSION: ${PYTHON3_VERSION}."
	@${ECHO} "Legal values are:"
	@${ECHO} "  python3.1"
	@${ECHO} "  python3.2 (default)"
	@${FALSE}
.endif

PYTHON3_MASTER_SITES=		${MASTER_SITE_PYTHON3}
PYTHON3_MASTER_SITE_SUBDIR=	ftp/python/${PYTHON3_PORTVERSION:C/rc[0-9]//}
PYTHON3_DISTFILE=			Python-${PYTHON3_PORTVERSION:S/.rc/rc/}.tgz
PYTHON3_WRKSRC=				${WRKDIR}/Python-${PYTHON3_PORTVERSION:S/.rc/rc/}

PYTHON3_INCLUDEDIR=		${PYTHON3BASE}/include/${PYTHON3_VERSION}
PYTHON3_LIBDIR=			${PYTHON3BASE}/lib/${PYTHON3_VERSION}
PYTHON3_PKGNAMEPREFIX=		py${PYTHON3_SUFFIX}-
PYTHON3_PKGNAMESUFFIX=		-py${PYTHON3_SUFFIX}
PYTHON3_PLATFORM=		${OPSYS:L}${OSREL:C/\.[0-9.]*//}
PYTHON3_SITELIBDIR=		${PYTHON3_LIBDIR}/site-packages

PYTHON3PREFIX_INCLUDEDIR=	${PYTHON3_INCLUDEDIR:S;${PYTHON3BASE};${PREFIX};}
PYTHON3PREFIX_LIBDIR=		${PYTHON3_LIBDIR:S;${PYTHON3BASE};${PREFIX};}
PYTHON3PREFIX_SITELIBDIR=	${PYTHON3_SITELIBDIR:S;${PYTHON3BASE};${PREFIX};}

# use PY3DISTUTILS
PY3DISTUTILS_CONFIGUREARGS?=
PY3DISTUTILS_BUILDARGS?=
PY3DISTUTILS_INSTALLARGS?=	-c -O1 --prefix=${PREFIX}
PY3DISTUTILS_PKGNAME?=		${PORTNAME}
PY3DISTUTILS_PKGVERSION?=	${PORTVERSION}
PY3DISTUTILS_EGGINFO?=		${PY3DISTUTILS_PKGNAME:C/[^A-Za-z0-9.]+/_/g}-${PY3DISTUTILS_PKGVERSION:C/[^A-Za-z0-9.]+/_/g}-${PYTHON3_VERSION:S/thon//}.egg-info
PY3DISTUTILS_EGGINFODIR?=	${PYTHON3PREFIX_SITELIBDIR}

.if !defined(PY3DISTUTILS_NOEGGINFO) && \
	(defined(INSTALLS_EGGINFO) ||  defined(USE_PY3DISTUTILS)) 
. for egg in ${PY3DISTUTILS_EGGINFO}
PLIST_FILES+=	${PY3DISTUTILS_EGGINFODIR:S;${PREFIX}/;;}/${egg}
. endfor
.endif

# Fix for programs that build python from a GNU auto* environment
CONFIGURE_ENV+=	PYTHON3="${PYTHON3_CMD}"

# Python 3rd-party modules
PY3DISTUTILS=	${PYTHON3_LIBDIR}/distutils/core.py:${PYTHON3_PORTSDIR}

# dependencies
PYTHON3_NO_DEPENDS?=		NO

.if ${PYTHON3_NO_DEPENDS} == "NO"
.if defined(USE_PYTHON3_BUILD)
BUILD_DEPENDS+=	${PYTHON3_CMD}:${PYTHON3_PORTSDIR}
.endif
.if defined(USE_PYTHON3_RUN)
RUN_DEPENDS+=	${PYTHON3_CMD}:${PYTHON3_PORTSDIR}
.endif
.endif		# ${PYTHON3_NO_DEPENDS} == "NO"

# set $PREFIX as Python's one
.if defined(USE_PYTHON3_PREFIX)
PREFIX=			${PYTHON3BASE}
.endif

# Substitutions for pkg-plist
# Use a short form of the PYTHON3PREFIX_*DIR variables; we don't need the
# base directory in the plist file.
PLIST_SUB+=	PYTHON3_INCLUDEDIR=${PYTHON3PREFIX_INCLUDEDIR:S;${PREFIX}/;;} \
		PYTHON3_LIBDIR=${PYTHON3PREFIX_LIBDIR:S;${PREFIX}/;;} \
		PYTHON3_PLATFORM=${PYTHON3_PLATFORM} \
		PYTHON3_SITELIBDIR=${PYTHON3PREFIX_SITELIBDIR:S;${PREFIX}/;;} \
		PYTHON3_VERSION=${PYTHON3_VERSION}

#.endif		# !defined(_POSTMKINCLUDED) && !defined(Python_Pre_Include)

#.if defined(_POSTMKINCLUDED) && !defined(Python_Post_Include)

#Python_Post_Include=		bsd.python3.mk

# py-distutils support
PY3SETUP?=			setup.py
PY3DISTUTILS_CONFIGURE_TARGET?=	config
PY3DISTUTILS_BUILD_TARGET?=	build
PY3DISTUTILS_INSTALL_TARGET?=	install

.if defined(USE_PY3DISTUTILS)
LDSHARED?=	${CC} -shared
MAKE_ENV+=	LDSHARED="${LDSHARED}"

.if !target(do-configure) && !defined(HAS_CONFIGURE) && !defined(GNU_CONFIGURE)
do-configure:
	@(cd ${BUILD_WRKSRC}; ${SETENV} ${MAKE_ENV} ${PYTHON3_CMD} ${PY3SETUP} ${PY3DISTUTILS_CONFIGURE_TARGET} ${PY3DISTUTILS_CONFIGUREARGS})
.endif

.if !target(do-build)
do-build:
	@(cd ${BUILD_WRKSRC}; ${SETENV} ${MAKE_ENV} ${PYTHON3_CMD} ${PY3SETUP} ${PY3DISTUTILS_BUILD_TARGET} ${PY3DISTUTILS_BUILDARGS})
.endif

.if !target(do-install)
do-install:
	@(cd ${INSTALL_WRKSRC}; ${SETENV} ${MAKE_ENV} ${PYTHON3_CMD} ${PY3SETUP} ${PY3DISTUTILS_INSTALL_TARGET} ${PY3DISTUTILS_INSTALLARGS})
.endif
.endif  # defined USE_PY3DISTUTILS

#.endif		# defined(_POSTMKINCLUDED) && !defined(Python_Post_Include)